module ConstraintLayout exposing (layout, satisfy, Constraint, ConstraintKind(..))

import List.Extra as LE
import Tuple exposing (first, second)
import Vector exposing (..)

type alias Index = Int

type alias Constraint = (ConstraintKind, Index, Index)

type ConstraintKind
  = MinDistance Float
  | MaxDistance Float
  | RightOf Float
  | Below Float
  | FixedPosition Vector
  | SameY

satisfy : List Vector -> Constraint -> List (Index,Vector)
satisfy positions (constraint,ia,ib)  =
  case (LE.getAt ia positions, LE.getAt ib positions) of
    (Just a, Just b) ->
      case constraint of
        MinDistance mind ->
          let
            d = distance a b
            n = sub b a |> normalise
            f = max 0 ((mind-d)/2)
            m = smul f n
          in
            [ (ia, neg m)
            , (ib, m)
            ]
            
        FixedPosition p -> [ (ia, sub p a)]

        RightOf d->
          let
            dx = (first a) - ((first b) - d)
            f = max 0 dx/2
          in
            [ (ia, (-f,0))
            , (ib, (f,0))
            ]

        Below d->
          let
            dy = (first a) - ((first b) - d)
            f = max 0 dy/2
          in
            [ (ia, (0,-f))
            , (ib, (0,f))
            ]

        SameY ->
          let
            dy = (second b) - (second a)
          in
            [ (ia, (0, dy/10))
            , (ib, (0,-dy/10))
            ]
        _ -> []
    _ -> []


layout : List Constraint -> List Vector -> List Vector
layout constraints positions =
  let
    f = 0.1
    changes = List.concatMap (satisfy positions) constraints
  in
    List.foldl (\(i,v) -> LE.updateAt i (add (smul f v))) positions changes